#include "c51\cpragma.h"                /* pragma-definitions */
#pragma code
/* DATE :              VERSION :      */

/**************************************************************************/
/* Copyright(c)                                                           */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Module             : COMMAND for V24                                   */
/* Project            :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Purpose            : This module is the c-part for command.            */
/*                                                                        */
/*                      language is 'ANSI-C-Standard'                     */
/*                                                                        */
/**************************************************************************/

#define COMMAND

#pragma EJECT

#include <reg52.h>                      /* special function register 80515  */
#include <ctype.h>                      /* character-functions  */
#include "c51\cmacdef.h"                /* macrodefinitions */
#include "c51\ram.h"
#include "c51\cglob.h"
#include "c51\cport.h"
#include "c51\iicslv.h"
#include "c51\sio_mod.h"
#include <c_file.h>            

/**************************************************************************/

#define FOR_SCREEN  1

/**************************************************************************/

extern void iic_input(void);
extern void sio_input(void);
extern void send_sio_byte(byte value);
extern bit pack_to_char(byte *outptr);
extern bit ascii_hex(byte *value);

/**************************************************************************/

bit no_net_used;

/**************************************************************************/

byte code error_msg[] = "\n*** ERROR: no valid command";
byte code hex_ascii[] = { '0','1','2','3','4','5','6','7',
                          '8','9','A','B','C','D','E','F', };

/**************************************************************************/

void command()
{
  no_net_used = 0;                      /* dummy for uncalled routines */
  if (no_net_used)
  {
    getchar();
    ungetchar(0x13);
  }

  if ((sio_message() != 0) &&           /* complete ascii sequenze with CR ... */
      !iic_send               )
  {                                     /* ... if buffer full then sio_message = 0xff */
    sio_input();
  }

  if (sio_outempty()    &&              /* To start sio-output, sio must be empty. */
     (iic_rx_size != 0)    )
  {
    iic_input();
  }
}

/**************************************************************************/

void iic_input(void)
{
byte cnt;

  putchar(SEQUENCE_START);
  --iic_rx_size;
  if(iic_rx_size != 0)
  {
    for (cnt = 0; cnt <= iic_rx_size; ++cnt)
    {
      send_sio_byte(iic_rx[cnt]);
    }
  }
  putchar(TERMINATOR);
  iic_rx_size = 0;
}

/**************************************************************************/

void sio_input(void)
{
byte value;
byte error;
word value_w;

  error = 0;
  value = getchar();

  if ((value == SEQUENCE_START) && sio_inrdy_to_pack())
  {
    iic_tx_size = 0;
    if (pack_to_char(&value))
    {
      switch(value)
      {
        case 0x01:
          while (sio_inrdy_to_pack())
          {
            if (pack_to_char(&value))
            {
              iic_tx[iic_tx_size] = value;
              ++iic_tx_size;
            }
            else
            {
              error = 1;
              break;
            }
          }
          if (!error)
          {
            if (getchar() == TERMINATOR)
            {
              iic_send = 1;
            }
            else
            {
              iic_tx_size = 0;
              error = 2;
            }
          }
        break;

        case 0x02:
          error = 3;
        break;

        default:
          error = 4;
        break;
      }
    }
    else
    {
      error = 5;
    }
  }
  else
  {
    if (value != TERMINATOR)
      while (sio_inrdy() && (getchar() != TERMINATOR)); /* ignor message */
//  error = 6;
  }

  if (error != 0)
  {
    while (sio_inrdy() && (getchar() != TERMINATOR)); /* ignor message */
//    printf("\n*** ERROR: no valid command %3n", error);
    printf(&error_msg[0]);
//    printf("%3n",error);
    printf(".\n");
  }
}

/**************************************************************************/

void send_sio_byte(byte value)
{
  putchar(hex_ascii[value >> 4]);
  putchar(hex_ascii[value & 0x0f]);
#if (FOR_SCREEN)
  putchar(BLANK);
#endif

}

/**************************************************************************/

bit pack_to_char(byte *outptr)
{
byte value;

  if ((value = getchar()) < BLANK)
    return (0);
  if (!ascii_hex(&value))               /* no valid numeric digit */
    return (0);
  *outptr = value << 4;
  if ((value = getchar()) < BLANK)
    return (0);
  if (!ascii_hex(&value))               /* no valid numeric digit */
    return (0);
  *outptr |= (value & 0x0f);
  return(1);
}

/**************************************************************************/

bit ascii_hex(byte *value)
{
  if (!(isalnum(*value)))
    return(0);
  *value = toupper(*value);
  if ((*value >= '0') && (*value <= '9'))
    *value -= 0x30;
  else
    *value -= 0x37;
  return(1);
}

/**************************************************************************/

