#include "c51\cpragma.h"                /* pragma-definitions */
#pragma code
/* DATE :               VERSION :      */

/**************************************************************************/
/* Copyright(c)                                                           */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Module             : IIC-SLAVE                                         */
/* Project            :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Purpose            : This module is the c-part to receive iic-bus      */
/*                                                                        */
/*                      language is 'ANSI-C-Standard'                     */
/*                                                                        */
/**************************************************************************/

#define IICSLV

#pragma EJECT

#include <reg52.h>                      /* special function register 80515  */
#include <intrins.h>                    /* special function */
#include "c51\cmacdef.h"                /* macrodefinitions */
#include "c51\ram.h"
#include "c51\cglob.h"
#include "c51\cport.h"
#include "c51\iicslv.h"

/**************************************************************************/

extern void iic_init(void);
extern void iic_give_acknowledge(void);
extern byte iic_read_acknowledge(void);
extern void iic_read_data(void);
extern void iic_write_data(byte iic_wr_data);

/**************************************************************************/
/*                                                                        */
/* Function           : IIC_INIT                                          */
/*                                                                        */
/* Project            :                                                   */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

void iic_init(void)
{
  EX0 = 1;                              /* enable S_INT-interrupt */
  PX0 = 1;                              /* interrupt highest priority */
  IT0 = 1;                              /* edge trigger */
  IE0 = 0;
  iic_send = 0;                         /* nothing to send */
  iic_rx_size = 0;                      /* nothing received */
  iic_tx_size = 0;                      /* nothing to send */
}


/**************************************************************************/
/*                                                                        */
/* Function           : IIC_SLAVE_INT                                     */
/*                                                                        */
/* Project            :                                                   */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

void iic_slave_int(void) interrupt iic_int using iic_bank
{
  EA = 0;                                 /* disable general interrupt */
  EX0 = 0;                                /* disable own interrupt */
  PX0 = 0;                                /* no more the highest priority */
  if (sda == 0)                           /* check sda high at beginning */
  {                                       /* iic_error ! */
    sda = 1;                              /* release bus lines */
    scl_low = 1;
    PX0 = 1;                              /* highest priority */
    EX0 = 1;                              /* enable own interrupt */
    EA = 1;                               /* enable general interrupt */
    return;
  }
  do
  {
    if (scl == 0)                         /* if scl goes low first -> error */
    {                                     /* iic_error ! */
      sda = 1;                            /* release bus lines */
      scl_low = 1;
      PX0 = 1;                            /* highest priority */
      EX0 = 1;                            /* enable own interrupt */
      EA = 1;                             /* enable general interrupt */
      return;
    }
  } while (sda == 1);                     /* wait for sda low */
  do
  {
    if (sda == 1)                         /* if sda goes high again -> error */
    {                                     /* iic_error ! */
      sda = 1;                            /* release bus lines */
      scl_low = 1;
      PX0 = 1;                            /* highest priority */
      EX0 = 1;                            /* enable own interrupt */
      EA = 1;                             /* enable general interrupt */
      return;
    }
  } while (scl == 1);                     /* wait for scl low */
  scl_low = 0;                            /* hold scl low */
  EA = 1;                                 /* enable interrupts */

  iic_start = 0;
  iic_stop = 0;

  while (!iic_stop)
  {
    iic_read_data();                      /* read slave address in iic_data */
    if ((iic_data == SLAVE_ADDRESS_RD) && /* functional data write address */
        (iic_rx_size == 0)                )
    {
      iic_give_acknowledge();
      iicptr = &iic_rx[0];
      iic_rx_size = 0xff;

      for ( ; ; )
      {
        iic_read_data();                  /* read functional data byte */
        ++iic_rx_size;                    /* byte-cnt, how much bytes are received */
        if (iic_stop)                     /* test stop condition */
          break;
        if (iic_start)                    /* test repeated start condition */
          break;
        *iicptr = iic_data;               /* save functional data */
        ++iicptr;                         /* increment address pointer */
        iic_give_acknowledge();
        if (iicptr > &iic_rx[iic_rx_buffersize])
        {
          iic_stop = 1;                   /* protect normal ram */
          break;
        }
      }
    }
    else
    {
      if ((iic_data == SLAVE_ADDRESS_WR) &&/* function data read address */
          iic_send                          )
      {
        iic_give_acknowledge();
        iicptr = &iic_tx[0];
        iic_send = 0;

        for ( ; ; )
        {
          iic_write_data(*iicptr);        /* write data to master */
          ++iicptr;                       /* increment address pointer */
          if (iic_read_acknowledge())
          {
            iic_stop = 1;                 /* finished if no acknowledge received */
            break;
          }
          --iic_tx_size;
          if (iic_tx_size == 0)           /* no more to send */
          {
            iic_stop = 1;                 /* protect normal ram */
            break;
          }
        }
      }
      else
      {
        iic_stop = 1;                     /* unknown address -> finished */
      }
    }
  }

  sda = 1;                                /* release bus lines */
  scl_low = 1;

  PX0 = 1;                                /* highest priority */
  EX0 = 1;                                /* enable own interrupt */
  EA = 1;                                 /* enable general interrupt */
}


/**************************************************************************/
/*                                                                        */
/* Function           : IIC_READ_DATA                                     */
/*                                                                        */
/* Project                                                                */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

void iic_read_data(void)
{
byte loop;

  sda = 1;                                /* sda out high for reading */
  iic_data = 0;                           /* clear A */
                                          /* read first data bit */
  EA = 0;                                 /* disable interrupts */
  scl_low = 1;                            /* release scl */
  while (scl == 0);                       /* wait for scl high */
  iic_data_0 = sda;                       /* read data bit */
  if (iic_data_0)
  {
    do
    {
      if (sda == 0)                       /* repeated start detection */
      {
        while (scl == 1);                 /* wait for scl low */
        scl_low = 0;                      /* hold scl - stretching */
        EA = 1;                           /* enable interrupts */
        iic_start = 1;
        return;
      }
    } while (scl == 1);                   /* wait for scl low */
  }
  else
  {
    do
    {
      if (sda == 1)                       /* stop condition detection */
      {
        iic_stop = 1;
        return;
      }
    } while (scl == 1);                   /* wait for scl low */
  }
  scl_low = 0;                            /* hold scl - stretching */
  EA = 1;                                 /* enable interrupts */
  loop = 7;                               /* read following 7 data bits */
  do
  {
    EA = 0;                               /* disable interrupts */
    iic_data = _crol_(iic_data,1);        /* rotate 1 byte, to move next bit in bit 0 */
    scl_low = 1;                          /* release scl */
    while (scl == 0);                     /* wait for scl high */
    iic_data_0 = sda;                     /* read data bit */
    while (scl == 1);                     /* wait for scl low */
    scl_low = 0;                          /* hold scl - stretching */
    EA = 1;                               /* enable interrupts */
  } while (--loop);                       /* loop for 7 data bits */
  return;
}


/**************************************************************************/
/*                                                                        */
/* Function           : IIC_GIVE_ACKNOWLEDGE                              */
/*                                                                        */
/* Project            :                                                   */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

void iic_give_acknowledge(void)
{
  sda = 0;                                /* sda out low for acknowledge */
  EA = 0;                                 /* disable interrupts */
  scl_low = 1;                            /* release scl */
  while (scl == 0);                       /* wait for scl high */
  while (scl == 1);                       /* wait for scl low */
  scl_low = 0;                            /* hold scl - stretching */
  EA = 1;                                 /* enable interrupts */
}


/**************************************************************************/
/*                                                                        */
/* Function           : IIC_WRITE_DATA                                    */
/*                                                                        */
/* Project            :                                                   */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

void iic_write_data(byte iic_wr_data)
{
byte loop;

  loop = 8;                             /* write 8 data bits */
  iic_data = iic_wr_data;
  do
  {
    EA = 0;                             /* disable interrupts */
    sda = iic_data_7;                   /* output data bit on sda */
    iic_data = _crol_(iic_data,1);      /* rotate 1 byte, to get next bit in bit 0 */
    scl_low = 1;                        /* release scl */
    while (scl == 0);                   /* wait for scl high */
    while (scl == 1);                   /* wait for scl low */
    scl_low = 0;                        /* hold scl - stretching */
    EA = 1;                             /* enable interrupts */
  } while (--loop);                     /* loop for 8 data bits */
}


/**************************************************************************/
/*                                                                        */
/* Function           : IIC_READ_ACKNOWLEDGE                              */
/*                                                                        */
/* Project            :                                                   */
/* Module             :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Precondition       :                                                   */
/*                                                                        */
/* Postcondition      :                                                   */
/*                                                                        */
/* Comment            :                                                   */
/*                                                                        */
/**************************************************************************/

byte iic_read_acknowledge(void)
{
  iic_data = 0;
  sda = 1;                              /* sda out high for reading */
  EA = 0;                               /* disable interrupts */
  scl_low = 1;                          /* release scl */
  while (scl == 0);                     /* wait for scl high */
  iic_data_0 = sda;                     /* read acknowledge bit */
  while (scl == 1);                     /* wait for scl low */
  scl_low = 0;                          /* hold scl - stretching */
  EA = 1;                               /* enable interrupts */
  return(iic_data);
}


/**************************************************************************/

