#include "c51\cpragma.h"                /* pragma-definitions */
#pragma code
/* DATE :               VERSION :      */

/**************************************************************************/
/* Copyright(c)                                                           */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Module             : MAINPART for V24                                  */
/* Project            :                                                   */
/* Creation date      :                                                   */
/* Author             :                                                   */
/* Change History     : Change date                     Changes made by   */
/*                                                                        */
/* Purpose            : This module is the c-part for mainprogram.        */
/*                                                                        */
/*                      language is 'ANSI-C-Standard'                     */
/*                                                                        */
/**************************************************************************/

#define MAIN

#pragma EJECT

#include <reg52.h>                      /* special function register 80515  */
#include <intrins.h>                    /* special function */
#include "c51\cmacdef.h"                /* macrodefinitions */
#include "c51\ram.h"
#include "c51\cglob.h"
#include "c51\cport.h"
#include "c51\sio_mod.h"
#include <c_file.h>                    

/**************************************************************************/

extern void command();
extern void init_reset();
extern void int_reti();
extern void iic_init();

/**************************************************************************/

extern void init_task_int(void);

/**************************************************************************/

#define UPPER(v)                (char)(v >> 8)
#define LOWER(v)                (char)v

#define t0_init_value           -1406

static byte task_cnt;
byte bdata task_byte1;
byte bdata task_byte2;
sbit t_12_1_bit = task_byte1 ^ 0;
sbit t_12_2_bit = task_byte1 ^ 1;
sbit t_25_1_bit = task_byte1 ^ 2;
sbit t_25_2_bit = task_byte1 ^ 3;
sbit t_50_1_bit = task_byte1 ^ 4;
sbit t_50_2_bit = task_byte1 ^ 5;
sbit t_100_1_bit = task_byte1 ^ 6;
sbit t_100_2_bit = task_byte1 ^ 7;
sbit t_100_3_bit = task_byte2 ^ 0;
sbit t_100_4_bit = task_byte2 ^ 1;
sbit t_100_5_bit = task_byte2 ^ 2;
sbit t_200_1_bit = task_byte2 ^ 3;
sbit t_200_2_bit = task_byte2 ^ 4;
sbit t_400_1_bit = task_byte2 ^ 5;
sbit t_400_2_bit = task_byte2 ^ 6;
sbit t_800_1_bit = task_byte2 ^ 7;

/**************************************************************************/

void main (void) using main_bank
{

/**************************************************************************/
/*-         timing of the initialisation of MM3 V24                       */
/*------------------------------------------------------------------------*/
/*                                                                        */
/*    1ms   8052 initialisation                                           */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
/*                                                                        */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
/*                                                                        */
/*      MAINLOOP !!!                                                      */
/**************************************************************************/


/* ====================================================================== */

  IE=0;                 /* ( EA  | --- | ET2 | ES  | ET1  | EX1 | ET0 | EX0   ) */
  IP=0;                 /* ( --- | --- | PT2 | PS  | PT1  | PX1 | PT0 | PX0   ) */

  int_reti();
  int_reti();           /* interrupt-in-progress-flags */

  P0=0xff;              /* set data & address */
  P2=0xff;
  P1=0xff;              /* set all port-pins high */
  P3=0x7f;              /* set all port-pins high, bank 0 */
  _nop_;
                        /* here are all flags of S
  PSW=0;                /* ( CY  | AC  | F0  | RS1 | RS0  | OV  | --- | P     ) */

  PCON=0;               /* ( SMOD| --- | --- | --- | GF1  | GF0 | PD  | IDL   ) */

  TMOD=0x00;            /* ( GATE| C/t | M1  | M0  | GATE | C/t | M1  | M0    ) */

  TCON=0x00;            /* ( TF1 | TR1 | TF0 | TR0 | IE1  | IT1 | IE0 | IT0   ) */

  TH0=UPPER(t0_init_value);
  TL0=LOWER(t0_init_value);

  SCON=0x00;            /* ( SM0 | SM1 | SM2 | REN | TB8  | RB8 | TI  | RI    ) */

  T2CON=0;              /* ( TF2 | EXF2| RCLK| TCLK| EXEN2| TR2 | C/T2| CP/RL2) */

  SP = stack - 1;       /* after reset chip bank=1, stack=stack+2 from 2*push !!! */

/* ====================================================================== */

  sio_init(SOFTFLOW);
  iic_init();
  init_task_int();

  EA = 1;




/*---------------------------*/
/*-       MAIN_LOOP         -*/
/*---------------------------*/

  for(;;)     /* endless = main_loop for main part */
  {
mainloop:
    PSW = 0x00;
    if(SP != (stack - 1))
    {
      init_reset();     /* software-reset !!! */
    }
    P0 = 0xff;          /* set data & address port high!!! */
    P2 = 0xff;          /* set address port high!!! */
    P1 |= 0x7f;         /* all portpins without bankswitch */
    P3 |= 0x01;         /* to read in the UART right! */
    scl_low = 1;
    PCON = 0x00;
/*
    IE = IE & 0x87;
    SCON = SCON & 0x1f;
    IE = IE | 0x87;      enable EA, ET0, EX0
    SCON = SCON | 0x10;  refresh mode 0 shift reg. mode & REN
    IP = 0x00;           no priority set
    TMOD = 0x11;         T0, T1 16 BIT timer refresh
    TR0 = 1;
*/

/* -------------------------------------- task_byte1  12.5ms            */
    if (t_12_1_bit)
    {

      t_12_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  12.5ms            */
    if (t_12_2_bit)
    {

      t_12_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  25ms              */
    if (t_25_1_bit)
    {

      t_25_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  25ms              */
    if (t_25_2_bit)
    {

      t_25_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  50ms              */
    if (t_50_1_bit)
    {
      command();
      t_50_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  50ms              */
    if (t_50_2_bit)
    {

      t_50_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  100ms             */
    if (t_100_1_bit)
    {

      t_100_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte1  100ms             */
    if (t_100_2_bit)
    {

      t_100_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  100ms             */
    if (t_100_3_bit)
    {

      t_100_3_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  100ms             */
    if (t_100_4_bit)
    {

      t_100_4_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  100ms             */
    if (t_100_5_bit)
    {

      t_100_5_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  200ms             */
    if (t_200_1_bit)
    {

      t_200_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  200ms             */
    if (t_200_2_bit)
    {


      t_200_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  400ms             */
    if (t_400_1_bit)
    {

      t_400_1_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  400ms             */
    if (t_400_2_bit)
    {

      t_400_2_bit = 0;
      continue;
    }

/* -------------------------------------- task_byte2  800ms             */
    if (t_800_1_bit)
    {

      t_800_1_bit = 0;
      continue;
    }
  }           /* endless = main_loop for main part */
}

/**************************************************************************/

void task_int(void) interrupt 1
{
byte task_help;

  TR0 = 0;
  TH0=UPPER(t0_init_value);
  TL0=LOWER(t0_init_value);
  TR0 = 1;

  task_help = task_cnt;
  if (task_help == 127)
  {                                     /* 800ms task */
    t_800_1_bit = 1;
  }
  task_help &= 0x7f;
  if (task_help == 63)
  {                                     /* 400ms task */
    t_400_1_bit = 1;
    t_400_2_bit = 1;
  }
  task_help &= 0x3f;
  if (task_help == 31)
  {                                     /* 200ms task */
    t_200_1_bit = 1;
    t_200_2_bit = 1;
  }
  task_help &= 0x1f;
  if (task_help == 15)
  {                                     /* 100ms task */
    t_100_1_bit = 1;
    t_100_2_bit = 1;
    t_100_3_bit = 1;
    t_100_4_bit = 1;
    t_100_5_bit = 1;
  }
  task_help &= 0x0f;
  if (task_help == 7)
  {                                     /* 50ms task */
    t_50_1_bit = 1;
    t_50_2_bit = 1;
  }
  task_help &= 0x07;
  if (task_help == 3)
  {                                     /* 25ms task */
    t_25_1_bit = 1;
    t_25_2_bit = 1;
  }
  task_help &= 0x03;
  if (task_help == 1)
  {                                     /* 12.5ms task */
    t_12_1_bit = 1;
    t_12_2_bit = 1;
  }
  ++task_cnt;
}

/**************************************************************************/

void init_task_int(void)
{
  TR0 = 0;
  TH0 = UPPER(t0_init_value);
  TL0 = LOWER(t0_init_value);
  ET0 = 1;
  task_cnt = 0;
  TR0 = 1;
}

/**************************************************************************/

